/*****************************************************************
 * Tool: SEAT - Software Exploration and Analysis Tool			 *
 *																 *
 * Class: TraceLoader.java										 *	
 * Package: io										 			 *
 *																 *
 * @author: Abdelwahab Hamou-Lhadj								 *
 *																 *	
 * Affiliation: Knowledge Based Reverse Engineering Group		 *
 *				SITE - University of Ottawa						 *
 *																 *
 * @version: 1.0 - July 2003									 *
 * 																 *
 * Modifications: 												 *
 * (please add date of modifications and author after this line) *	
 *																 *
 *****************************************************************/
 
package seat.io;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;

import seat.model.core.dynamic.Edge;
import seat.model.core.dynamic.MethodInvocationNode;
import seat.model.core.dynamic.Node;
import seat.model.core.dynamic.Trace;
import seat.model.core.dynamic.TreeTrace;
import seat.model.core.structure.BehaviouralElement;
import seat.model.core.structure.ClassElement;
import seat.model.core.structure.MethodElement;
import seat.model.core.structure.SoftwareSystem;
import seat.model.core.structure.StreamSource;



public class MethodInvocationTraceLoader extends TraceLoader
{
  private Hashtable labels;
  private Hashtable nodes;
  private Hashtable classes;
  private BufferedReader traceReader;
  private String ln;

  /** Creates a method invocation trace loader
   *
   * @param trace the trace to load
   *
   */
    
  public MethodInvocationTraceLoader(Trace trace)
  {
	 super(trace);
  	 
	 labels = new Hashtable();
	 nodes  = new Hashtable();
	 classes= new Hashtable();
	 traceReader = null;
	 ln = "";
  }
  
  /** Loads a trace into memory. */

  public void open() 
  {
	try 
	{
	   
	   openTrace(); // open the file
	   skip(); // skips comments and empty lines
	   
	   if (ln.startsWith("#Labels")) 
	   {
		  skip();
  	  	  
		  // loading labels
		  while (!ln.startsWith("#Nodes")) 
		  {
			//System.out.println("add " + ln);
			addLabel();
			skip();
		  }
  	  	  
		  skip();
  	  	  
		  // loading nodes
		  while (!ln.startsWith("#Edges")) 
		  {
			//System.out.println("node " + ln);
			addNode();
			skip();
		  }
  	  	  
		  skip();

		  while (ln != null)
		  {
			//System.out.println("edge " + ln);
			addEdge();
			skip();
		  }
	   }	
	   
	   traceReader.close();
	   
	   	
	   /* The file was read
		* we need to determine the root of the trace
		*/
		 
	   for (Enumeration e = nodes.keys(); e.hasMoreElements();)
	   {
		  Node node = (Node)nodes.get((String)e.nextElement());
		  if (node.isRoot())
		  {
			((TreeTrace)trace).setRoot(node);
			break;
		  }	
	   }
	} 
	catch (IOException e) 
	{
		System.out.println("Problem opening the CTF file");
	}
	catch (IndexOutOfBoundsException e)
	{
		System.out.println("Problem in line " + ln);
	}
	catch (NullPointerException e)
	{
		System.out.println("Problem in line " + ln);
	}
	catch (NumberFormatException e)
	{
		System.out.println("Problem in line " + ln);
	}
	catch (Exception e)
	{
		System.out.println("Problem in line " + ln);
	}
  }
 
  /** Adds a label 
   *
   * @throws IndexOutOfBoundsException
   * @throws NumberFormatException
   * @throws NullPointerException
   **/

  private void addLabel()
	throws IndexOutOfBoundsException, NullPointerException, NumberFormatException
  {
	 ClassElement cl;
	 BehaviouralElement behElem;
	 String label;
	 String labelID;  

	 label   = ln.substring(0, ln.indexOf(",")).trim(); // the label
	 labelID = ln.substring(ln.indexOf(",") + 1).trim(); // label ID
	 cl = null;
	 behElem = null;
	 
	 SoftwareSystem sys = trace.getScenario().getSystem();
	 if (sys.isParsed())
	 {
		cl = (ClassElement)sys.getModelElement(getString(label, 0));
		behElem = (BehaviouralElement)cl.getMethod(getString(label, 1));
	 }
	 else
	{
		// Modified by Eric to cache existent ClassElement
		// Note by Eric for parsing:
		// package string can be parsed from name of cl, which is a full name includes
		// pak1.pak2.ClassName,
		String fullName=getString(label, 0);
		cl= (ClassElement)classes.get(fullName);
		if (cl == null) {
			cl = new ClassElement(sys, fullName);
			classes.put(fullName, cl);
		}				
		behElem = new MethodElement(sys, getString(label, 1), cl);
		
//orginal
//		cl = new ClassElement(sys, getString(label, 0));
//		behElem = new MethodElement(sys, getString(label, 1), cl);
	 }
	 // Attention: The system might not be parsed well. What to do?
	 labels.put(labelID, behElem);
  }
  
  /** Adds a node 
   *
   * @throws IndexOutOfBoundsException
   * @throws NumberFormatException
   * @throws NullPointerException
   **/
   
  private void addNode()
	throws IndexOutOfBoundsException, NullPointerException, NumberFormatException
  {
	 String cert;    
	 String labelID; 
	 
	 cert = ln.substring(0, ln.indexOf(",")).trim(); // the certificate
	 labelID = ln.substring(ln.indexOf(",") + 1).trim(); // label ID
	 
	 Node node = new MethodInvocationNode((BehaviouralElement)labels.get(labelID), 
										  Integer.parseInt(cert));
	 nodes.put(cert, node);
	 
	 // update the comprehension units table of the trace
	 ((TreeTrace)trace).addComprehensionUnit(node, Integer.parseInt(cert));
  }

  /** Adds an edge 
   * 
   * @throws IndexOutOfBoundsException
   * @throws NumberFormatException
   * @throws NullPointerException
   **/
   
  private void addEdge()
	throws IndexOutOfBoundsException, NumberFormatException, NullPointerException
  {
	 StringTokenizer allTokens;
	 String cert1;
	 String cert2;
	 
     
	 allTokens = new StringTokenizer(ln, ",");
	 cert1 = (String)allTokens.nextToken().trim(); // the certificate 1
	 cert2 = (String)allTokens.nextToken().trim(); // the certificate 2

	 Node node1 = (Node)nodes.get(cert1);
	 Node node2 = (Node)nodes.get(cert2);
	 
	 node1.addChild(node2);
	 node2.addParent(node1);

	 // Look for repetitions due to loop
	 String repet = null;
	 if (allTokens.hasMoreTokens())
	 {
		repet = (String)allTokens.nextToken().trim();
		Edge edge = node1.getEdge(node2);
		edge.setNumberOfRepetitions(Integer.parseInt(repet));
	 }	
  }
  
  /** parse the trace file line by line: file_name routine_name nesting_level. */
  private String getString(String ln, int pos)
  {
	StringTokenizer allTokens = new StringTokenizer(ln, " ");
	String[] list = new String[allTokens.countTokens()];
	int i = 0;
	while(allTokens.hasMoreTokens())
	{
		list[i] = allTokens.nextToken();
		i++;
	}		
	return list[pos];
  }
  	
  /** Returns True if the line is empty. */
  private boolean isEmptyLine(String ln)
  {
	 return (ln.length() == 0);
  }
	
  /** Returns True if the line is a comment. */
  
  private boolean isComment(String ln)
  {
	return ln.startsWith("%");
  }
  
  /** Returns True if the line is a command. */
  
  private boolean isCommand(String ln)
  {
	return ln.startsWith("#");
  }
  
  /** Opens the file that corresponds to the trace. */
  
  // Modified on may5,2004 to load from a Source not a String, original name: openFile  
  private void openTrace()
	throws IOException
  {
	// Open the CTF physical file
	ITraceSource source = trace.getTraceSource(); // added on May5,2004
	if (source instanceof StreamSource)
		traceReader = new BufferedReader(new InputStreamReader(((StreamSource)source).getStream()));// added on May5,2004
	else 
		throw new IOException();
  }  
  
  /** Returns True if the file is empty. */
  
  private boolean EOF()
  {
	return (ln == null);
  }
  
  /** Returns the next element of the file. */
  
  private String getNext()
	throws IOException
  {
	 return traceReader.readLine();
  }
  
  /** Skips comments and empty lines. */
  
  private void skip()
	throws IOException
  {
	ln = getNext();
	if (!EOF())
	{
		ln = ln.trim();	
		while(isComment(ln) || isEmptyLine(ln)) 
			ln = getNext();
	 }		
  }

	// Counter for edges removed. edges!=total nodes.
	// Added by Eric to ask for state info from loader
	public Collection getNodes() {
		return nodes.values();
	}
	public int getUniuqeNodes() {
		return nodes.size();
	}
}
