/*
 * Created on Aug 5, 2003
 *
 * This class is the representation of each algorithm contribution in the plugin.xml
 * configuration file. Actual implentation of an alogirthm .class will be created 
 * through this description. 
 * It also contains a static method to find all the algorithms
 * that are presented in the plugin.xml.
 * 
 * @author lfu
 */
package seat.ui.algorithm;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IPluginRegistry;
import org.eclipse.core.runtime.Platform;

import seat.model.algorithms.Algorithm;

public class AlgorithmDescription {
	private IConfigurationElement algorithm; // a subnode of algorithm in plugin.xml
	
	/**
	 * Find all the configured algorithms in the plugin configuration.
	 * @param extpntId -The extension point id that represents the algorithm 
	 * @return an array of AlgorithmDescription.
	 */
	public static AlgorithmDescription[] findAll(String extpntId) {
		IPluginRegistry registry=Platform.getPluginRegistry();
		IExtensionPoint extPoint=registry.getExtensionPoint(extpntId);
		if (extPoint==null) return new AlgorithmDescription[0];
		IConfigurationElement[] elements=extPoint.getConfigurationElements();
		AlgorithmDescription[] algorithms=new AlgorithmDescription[elements.length];
		for (int i=0;i<elements.length;i++) 
			algorithms[i]= new AlgorithmDescription(elements[i]);
		return algorithms; 
	}

	/**
	 * Constructor. Using  result information from xml file parsing 
	 */
	public AlgorithmDescription( IConfigurationElement alg) {
		algorithm=alg; 
	}

	/**
	 * Creates an Algorithm object that this AlgorithmDescription stands for.
	 * Once the algorithm object is created, it can be used to filter in TreeView.
	 * @return- the algorithm created, or null if cannot.
	 */
	public Algorithm createAlgorithm() {
		Object alg=null;
		try {
			alg=algorithm.createExecutableExtension("class");
		} catch (CoreException ce) {
			ce.printStackTrace();
		}
		if ((alg!=null)&&(alg instanceof Algorithm)) 
			return (Algorithm)alg;
		else 
			return null;
	}

	/**
	 * Gets the id of algorithm in plugin.xml
	 * @return id
	 */
	public String getID() {
		return algorithm.getAttribute("id");
	}

	public String getIndex() {
		return algorithm.getAttribute("index");
	}
	
	/**
	 * GEts the name of algorithm in plugin.xml
	 * @return name
	 */
	public String getName() {
		return algorithm.getAttribute("name");
	}
	
	/**
	 * Gets the class the implements the actual algorithm in plugin.xml
	 * @return clas name
	 */
	public String getClassName() {
		return algorithm.getAttribute("class");
	}
	
	/**
	 * Returns the alogirthm category: OO, Routine or both
	 * @return String category
	 */
	public String getCategory() {
		return algorithm.getAttribute("category");		
	}

	/**
	 * Returns the icon name in default "icons" sub directory
	 * @return
	 */
	public String getIcon() {
		return algorithm.getAttribute("icon");		
	}

	
	/**
	 * Returns the description of algorithm in plugin.xml
	 * @return description
	 */
	public String getDescription() {
		IConfigurationElement[] children=algorithm.getChildren("description");
		String description="";
		if (children!=null) {
			for (int i=0;i<children.length;i++)
				description+=children[i].getValue();
		}
		return description;
	}
	
	/**
	 * Returns if the algoirthm needs special user interface.
	 * If true, user has to provide interface classes by extending Eclipse
	 * Otherwise, it will be loaded uniformly
	 * @return false: no special treatment, load uniformly.
	 */
	public boolean getSpecialUI() {
		String special=algorithm.getAttribute("specialUI");
		if (special!=null)
			return (special.equalsIgnoreCase("true"))?true:false;
		else 
			return false;		
	}
	
	/**
	 * Returns an array of Parameters of algorithm in plugin.xml
	 * Parameter objects are used to setup prefernce pages in UI.
	 * @return Parameter[]
	 * @see Parameter
	 */
	//?????????? current data or default???
	public Parameter[] getParameters() {
		IConfigurationElement[] children=algorithm.getChildren("parameter");
		Parameter[] paras=null;
		if (children!=null) {
			paras=new Parameter[children.length+1]; // reserved 1 for index
			paras[0]=new Parameter("index","index","internal",getIndex());
			for (int i=0;i<children.length;i++) {
				String name=children[i].getAttribute("name");
				String id=children[i].getAttribute("id");
				String type=children[i].getAttribute("type");
				String def=children[i].getAttribute("default");
				if ((name==null)||(id==null)||(type==""))
					paras[i+1]=new Parameter("","","","");
				else {
					if (def==null)
						def="";
					paras[i+1]=new Parameter(name,id,type,def);
				}
			}
		}
		return paras;
	}
				
}
